=== modified file 'lib/common/console.py'
--- lib/common/console.py	2008-08-19 06:44:05 +0000
+++ lib/common/console.py	2008-08-20 08:10:40 +0000
@@ -27,6 +27,7 @@
 import os
 import random
 import socket
+import StringIO
 import uuid
 
 import cjson
@@ -56,6 +57,68 @@
     def __str__(self):
         return repr(self.value)
 
+class TruncateStringIO(StringIO.StringIO):
+    """ A class that wraps around StringIO and truncates the buffer when the 
+    contents are read (except for when using getvalue).
+    """
+    def __init__(self, buffer=None):
+        StringIO.StringIO.__init__(self, buffer)
+    
+    def read(self, n=-1):
+        """ Read at most size bytes from the file (less if the read hits EOF 
+        before obtaining size bytes).
+
+        If the size argument is negative or omitted, read all data until EOF is      
+        reached. The bytes are returned as a string object. An empty string is 
+        returned when EOF is encountered immediately.
+
+        Truncates the buffer.
+        """
+
+        self.seek(0)
+        res = StringIO.StringIO.read(self, n)
+        self.truncate(0)
+        return res
+
+    def readline(self, length=None):
+        """ Read one entire line from the file.
+ 
+        A trailing newline character is kept in the string (but may be absent 
+        when a file ends with an incomplete line). If the size argument is   
+        present and non-negative, it is a maximum byte count (including the      
+        trailing newline) and an incomplete line may be returned.
+
+        An empty string is returned only when EOF is encountered immediately.
+        
+        Note: Unlike stdio's fgets(), the returned string contains null   
+        characters ('\0') if they occurred in the input.
+
+        Removes the line from the buffer.
+        """
+
+        self.seek(0)
+        res = StringIO.StringIO.readline(self, length)
+        rest = StringIO.StringIO.read(self)
+        self.truncate(0)
+        self.write(rest)
+        return res
+
+    def readlines(self, sizehint=0):
+        """ Read until EOF using readline() and return a list containing the        
+        lines thus read.
+        
+        If the optional sizehint argument is present, instead of reading up to 
+        EOF, whole lines totalling approximately sizehint bytes (or more to      
+        accommodate a final whole line).
+
+        Truncates the buffer.
+        """
+
+        self.seek(0)
+        res = StringIO.StringIO.readlines(self, length)
+        self.truncate(0)
+        return res
+
 class Console(object):
     """ Provides a nice python interface to the console
     """
@@ -66,9 +129,21 @@
         self.uid = uid
         self.jail_path = jail_path
         self.working_dir = working_dir
+
+        # Set up the buffers
+        self.stdin = TruncateStringIO()
+        self.stdout = TruncateStringIO()
+        self.stderr = TruncateStringIO()
+
+        # Fire up the console
         self.restart()
 
     def restart(self):
+        # Empty all the buffers
+        self.stdin.truncate(0)
+        self.stdout.truncate(0)
+        self.stderr.truncate(0)
+
         # TODO: Check if we are already running a console. If we are shut it 
         # down first.
 
@@ -139,10 +214,27 @@
             raise ConsoleError(
                 "Could not understand the python console response")
 
-        # Look for user errors
+        return response
+
+    def __handle_chat(self, cmd, args):
+        """ A wrapper around self.__chat that handles all the messy responses 
+        of chat for higher level interfaces such as inspect
+        """
+        # Do the request
+        response = self.__chat(cmd, args)
+
+        # Process I/O requests
+        while 'output' in response or 'input' in response:
+            if 'output' in response:
+                self.stdout.write(response['output'])
+                response = self.chat()
+            elif 'input' in response:
+                response = self.chat(self.stdin.readline())
+
+        # Process user exceptions
         if 'exc' in response:
             raise ConsoleException(response['exc'])
-        
+
         return response
 
     def chat(self, code=''):
@@ -151,7 +243,7 @@
 
     def block(self, code):
         """ Executes a block of code and returns the output """
-        block = self.__chat('block', code)
+        block = self.__handle_chat('block', code)
         if 'output' in block:
             return block['output']
         elif 'okay' in block:
@@ -159,21 +251,25 @@
         else:
             raise ConsoleException("Bad response from console: %s"%str(block))
 
-    def flush(self, globs=None):
-        """ Resets the consoles globals() to the default and optionally augment 
-        them with a dictionary simple globals. (Must be able to be pickled)
+    def globals(self, globs=None):
+        """ Returns a dictionary of the console's globals and optionally set 
+        them to a new value
         """
         # Pickle the globals
-        pickled_globs = {}
+        pickled_globs = None
         if globs is not None:
+            pickled_globs = {}
             for g in globs:
                 pickled_globs[g] = cPickle.dumps(globs[g])
 
-        flush = self.__chat('flush', pickled_globs)
-        if 'response' in flush and flush['response'] == 'okay':
-            return
-        else:
-            raise ConsoleError("Bad response from console: %s"%str(flush))
+        globals = self.__handle_chat('globals', pickled_globs)
+
+        # Unpickle the globals
+        for g in globals['globals']:
+            globals['globals'][g] = cPickle.loads(globals['globals'][g])
+
+        return globals['globals']
+        
 
     def call(self, function, *args, **kwargs):
         """ Calls a function in the python console. Can take in a list of 
@@ -184,8 +280,8 @@
             'function': function,
             'args': args,
             'kwargs': kwargs}
-        call = self.__chat('call', call_args)
-        
+        call = self.__handle_chat('call', call_args)
+
         # Unpickle any exceptions
         if 'exception' in call:
             call['exception']['except'] = \
@@ -193,26 +289,18 @@
 
         return call
 
-    def inspect(self, code=''):
-        """ Runs a block of code in the python console returning a dictionary 
-        summary of the evaluation. Currently this includes the values of 
-        stdout, stderr, simple global varibles.
-        If an exception was thrown then this dictionary also includes a 
-        exception dictionary containg a traceback string and the exception 
-        except.
+    def execute(self, code=''):
+        """ Runs a block of code in the python console.
+        If an exception was thrown then returns an exception object.
         """
-        inspection = self.__chat('inspect', code)
-       
-        # Unpickle the globals
-        for g in inspection['globals']:
-            inspection['globals'][g] = cPickle.loads(inspection['globals'][g])
-        
+        execute = self.__handle_chat('execute', code)
+              
         # Unpickle any exceptions
-        if 'exception' in inspection:
-            inspection['exception']['except'] = \
-                cPickle.loads(inspection['exception']['except'])
+        if 'exception' in execute:
+            return cPickle.loads(execute['exception'])
+        else:
+            return execute
 
-        return inspection
 
     def set_vars(self, variables):
         """ Takes a dictionary of varibles to add to the console's global 
@@ -223,13 +311,13 @@
         for v in variables:
             vars[v] = repr(variables[v])
 
-        set_vars = self.__chat('set_vars', vars)
+        set_vars = self.__handle_chat('set_vars', vars)
 
         if set_vars.get('response') != 'okay':
             raise ConsoleError("Could not set variables")
 
     def close(self):
         """ Causes the console process to terminate """
-        self.__chat('restart', None)
+        return self.__chat('terminate', None)
     
 

=== modified file 'scripts/python-console'
--- scripts/python-console	2008-08-20 00:04:12 +0000
+++ scripts/python-console	2008-08-20 08:10:40 +0000
@@ -171,7 +171,6 @@
 
     def run(self):
         self.globs = {}
-        self.globs['__builtins__'] = globals()['__builtins__']
         self.curr_cmd = ''
 
         while True:
@@ -205,24 +204,23 @@
                     self.webio.flush()
                     self.cmdQ.put({"exc": ''.join(tb).decode('utf-8', 'replace')})
                     self.curr_cmd = ''
-            elif 'flush' in ln:
-                # reset the globals
-                self.globs = {}
-                self.globs['__builtins__'] = globals()['__builtins__']
-                self.cmdQ.put({'response': 'okay'})
+            elif 'globals' in ln:
                 # Unpickle the new space (if provided)
-                if isinstance(ln['flush'],dict):
-                    for g in ln['flush']:
+                if isinstance(ln['globals'],dict):
+                    self.globs = {}
+                    for g in ln['globals']:
                         try:
-                            self.globs[g] = cPickle.loads(ln['flush'][g])
+                            self.globs[g] = cPickle.loads(ln['globals'][g])
                         except:
                             pass
+
+                # Return the current globals
+                self.cmdQ.put({'globals': flatten(self.globs)})
             elif 'call' in ln:
                 call = {}
-                stdout = cStringIO.StringIO()
-                stderr = cStringIO.StringIO()
-                sys.stdout = stdout
-                sys.stderr = stderr
+                sys.stdin = self.webio
+                sys.stdout = self.webio
+                sys.stderr = self.webio
 
                 if isinstance(ln['call'], dict):
                     params = ln['call']
@@ -258,43 +256,33 @@
                             {"exc": ''.join(tb).decode('utf-8', 'replace')})
                     
                     # Write out the inspection object
-                    call['stdout'] = stdout.getvalue()
-                    call['stderr'] = stderr.getvalue()
                     self.cmdQ.put(call)
                 else:
                     self.cmdQ.put({'response': 'failure'})
-                stdout.close()
-                stderr.close()
                 self.curr_cmd = ''
-            elif 'inspect' in ln:
+            elif 'execute' in ln:
                 # Like block but return a serialization of the state
                 # throw away partial command
-                inspection = {}
-                stdout = cStringIO.StringIO()
-                stderr = cStringIO.StringIO()
+                response = {'okay': None}
+                sys.stdin = self.webio
+                sys.stdout = self.webio
+                sys.stderr = self.webio
                 try:
-                    cmd = compile(ln['inspect'], "<web session>", 'exec');
-                    sys.stdin = None
-                    sys.stdout = stdout
-                    sys.stderr = stderr
+                    cmd = compile(ln['execute'], "<web session>", 'exec');
                     # We don't expect a return value - 'single' symbol prints 
                     # it.
                     self.eval(cmd)
                 except Exception, e:
-                    exception = {}
-                    tb = format_exc_start(start=1)
-                    exception['traceback'] = \
-                        ''.join(tb).decode('utf-8', 'replace')
-                    exception['except'] = cPickle.dumps(e, PICKLEVERSION)
-                    inspection['exception'] = exception                
+                    response = {'exception': cPickle.dumps(e, PICKLEVERSION)}
+               
+                # Flush the files
+                sys.stderr.flush()
+                sys.stdout.flush()
                 
-                # Write out the inspection object
-                inspection['stdout'] = stdout.getvalue()
-                inspection['stderr'] = stderr.getvalue()
-                inspection['globals'] = flatten(self.globs)
-                self.cmdQ.put(inspection)
-                stdout.close()
-                stderr.close()
+                # Return the inspection object
+                self.cmdQ.put(response)
+
+                # Clear any previous command
                 self.curr_cmd = ''
             elif 'set_vars' in ln:
                 # Adds some variables to the global dictionary
@@ -306,7 +294,7 @@
                         self.cmdQ.put(
                             {"exc": ''.join(tb).decode('utf-8', 'replace')})
 
-                self.cmdQ.put({'response': 'okay'})
+                self.cmdQ.put({'okay': None})
             else:
                 raise Exception, "Invalid Command"
 
@@ -349,14 +337,14 @@
 
 def dispatch_msg(msg):
     global terminate
-    if msg['cmd'] == 'restart':
-        terminate = "User requested console be reset"
+    if msg['cmd'] == 'terminate':
+        terminate = "User requested console be terminated"
     if terminate:
-        raise common.chat.Terminate({"restart":terminate})
+        raise common.chat.Terminate({"terminate":terminate})
     expiry.ping()
     lineQ.put({msg['cmd']:msg['text']})
     if terminate:
-        raise common.chat.Terminate({"restart":terminate})
+        raise common.chat.Terminate({"terminate":terminate})
     return cmdQ.get()
 
 def format_exc_start(start=0):

=== modified file 'www/apps/consoleservice/__init__.py'
--- www/apps/consoleservice/__init__.py	2008-08-13 07:26:44 +0000
+++ www/apps/consoleservice/__init__.py	2008-08-20 08:10:40 +0000
@@ -124,9 +124,9 @@
             # Could not decode the reply from the python-console server
             decoded_response = {"restart":
                 "Communication to console process lost"}
-        if "restart" in decoded_response:
+        if "terminate" in decoded_response:
             response = restart_console(uid, jail_path, working_dir,
-                decoded_response["restart"])
+                decoded_response["terminate"])
 
     except socket.error, (enumber, estring):
         if enumber == errno.ECONNREFUSED:

=== modified file 'www/apps/tutorialservice/test/TestFramework.py'
--- www/apps/tutorialservice/test/TestFramework.py	2008-08-19 06:44:05 +0000
+++ www/apps/tutorialservice/test/TestFramework.py	2008-08-20 08:10:40 +0000
@@ -23,10 +23,10 @@
 # Brief description of the Module# define custom exceptions
 # use exceptions for all errors found in testing
 
-import sys, StringIO, copy
+import sys, copy
 import types
 
-from common import util
+from common import testfilespace
 
 # student error or author error
 # errors in student code get handled internally
@@ -330,7 +330,7 @@
         if stdin[-1:] != '\n': stdin += '\n'
         
         self._stdin = stdin
-        self._filespace = TestFilespace(filespace)
+        self._filespace = testfilespace.TestFilespace(filespace)
         self._global_space = global_space
         self._parts = []
         self._allowed_exceptions = set()
@@ -465,7 +465,7 @@
         """
         self._initialise_global_space(global_space)
         
-        inspection = self._console.inspect(string)
+        inspection = self._console.execute(string)
 
         exception_name = None
         if 'exception' in inspection:
@@ -475,17 +475,17 @@
 
         return {'code': string,
                 'result': None,
-                'globals': inspection['globals'],
+                'globals': self._console.globals(),
                 'exception': exception_name, # Hmmm... odd? Is this right?
-                'stdout': inspection['stdout'],
-                'stderr': inspection['stderr'],
+                'stdout': self._console.stdout.read(),
+                'stderr': self._console.stderr.read(),
                 'modified_files': None}
 
     def _initialise_global_space(self, global_space):
         """ Modify the provided global_space so that file, open and raw_input are redefined
         to use our methods instead.
         """
-        self._console.flush(global_space)
+        self._console.globals(global_space)
         self._current_filespace_copy = self._filespace.copy()
         global_space['file'] = lambda filename, mode='r', bufsize=-1: self._current_filespace_copy.openfile(filename, mode)
         global_space['open'] = global_space['file']
@@ -594,202 +594,6 @@
     def get_name(self):
         return self._name
 
-class TestFilespace:
-    """
-    Our dummy file system which is accessed by code being tested.
-    Implemented as a dictionary which maps filenames to strings
-    """
-    def __init__(self, files=None):
-        "Initialise, optionally with filename-filedata pairs"
-
-        if files == None:
-            files = {}
-
-        # dict mapping files to strings
-        self._files = {}
-        self._files.update(files)
-        # set of file names
-        self._modified_files = set([])
-        # dict mapping files to stringIO objects
-        self._open_files = {}
-
-    def add_file(self, filename, data):
-        " Add a file to the filespace "
-        self._files[filename] = data
-
-    def openfile(self, filename, mode='r'):
-        """ Open a file from the filespace with the given mode.
-        Return a StringIO subclass object with the file contents.
-        """
-        # currently very messy, needs to be cleaned up
-        # Probably most of this should be in the initialiser to the TestStringIO
-        
-        import re
-
-        if filename in self._open_files:
-            raise IOError("File already open: %s" %filename)
-
-        if not re.compile("[rwa][+b]{0,2}").match(mode):
-            raise IOError("invalid mode %s" %mode)
-        
-        ## TODO: validate filename?
-        
-        mode.replace("b",'')
-        
-        # initialise the file properly (truncate/create if required)
-        if mode[0] == 'w':
-            self._files[filename] = ''
-            self._modified_files.add(filename)
-        elif filename not in self._files:
-            if mode[0] == 'a':
-                self._files[filename] = ''
-                self._modified_files.add(filename)
-            else:
-                raise IOError(2, "Access to file denied: %s" %filename)
-
-        # for append mode, remember the existing data
-        if mode[0] == 'a':
-            existing_data = self._files[filename]
-        else:
-            existing_data = ""
-
-        # determine what operations are allowed
-        reading_ok = (len(mode) == 2 or mode[0] == 'r')
-        writing_ok = (len(mode) == 2 or mode[0] in 'wa')
-
-        # for all writing modes, start off with blank file
-        if mode[0] == 'w':
-            initial_data = ''
-        else:
-            initial_data = self._files[filename]
-
-        file_object = TestStringIO(initial_data, filename, self, reading_ok, writing_ok, existing_data)
-        self._open_files[filename] = file_object
-        
-        return file_object
-
-    def flush_all(self):
-        """ Flush all open files
-        """
-        for file_object in self._open_files.values():
-            file_object.flush()
-
-    def updatefile(self,filename, data):
-        """ Callback function used by an open file to inform when it has been updated.
-        """
-        if filename in self._open_files:
-            self._files[filename] = data
-            if self._open_files[filename].is_modified():
-                self._modified_files.add(filename)
-        else:
-            raise IOError(2, "Access to file denied: %s" %filename)
-
-    def closefile(self, filename):
-        """ Callback function used by an open file to inform when it has been closed.
-        """
-        if filename in self._open_files:
-            del self._open_files[filename]
-
-    def get_modified_files(self):
-        """" A subset of the filespace containing only those files which have been
-        modified
-        """
-        modified_files = {}
-        for filename in self._modified_files:
-            modified_files[filename] = self._files[filename]
-
-        return modified_files
-
-    def get_open_files(self):
-        " Return the names of all open files "
-        return self._open_files.keys()
-            
-    def copy(self):
-        """ Return a copy of the current filespace.
-        Only the files are copied, not the modified or open file lists.
-        """
-        self.flush_all()
-        return TestFilespace(self._files)
-
-class TestStringIO(StringIO.StringIO):
-    """
-    A subclass of StringIO which acts as a file in our dummy file system
-    """
-    def __init__(self, string, filename, filespace, reading_ok, writing_ok, existing_data):
-        """ Initialise with the filedata, file name and infomation on what ops are
-        acceptable """
-        StringIO.StringIO.__init__(self, string)
-        self._filename = filename
-        self._filespace = filespace
-        self._reading_ok = reading_ok
-        self._writing_ok = writing_ok
-        self._existing_data = existing_data
-        self._modified = False
-        self._open = True
-
-    # Override all standard file ops. Make sure that they are valid with the given
-    # permissions and if so then call the corresponding method in StringIO
-    
-    def read(self, *args):
-        if not self._reading_ok:
-            raise IOError(9, "Bad file descriptor")
-        else:
-            return StringIO.StringIO.read(self, *args)
-
-    def readline(self, *args):
-        if not self._reading_ok:
-            raise IOError(9, "Bad file descriptor")
-        else:
-            return StringIO.StringIO.readline(self, *args)
-
-    def readlines(self, *args):
-        if not self._reading_ok:
-            raise IOError(9, "Bad file descriptor")
-        else:
-            return StringIO.StringIO.readlines(self, *args)
-
-    def seek(self, *args):
-        if not self._reading_ok:
-            raise IOError(9, "Bad file descriptor")
-        else:
-            return StringIO.StringIO.seek(self, *args)
-
-    def truncate(self, *args):
-        self._modified = True
-        if not self._writing_ok:
-            raise IOError(9, "Bad file descriptor")
-        else:
-            return StringIO.StringIO.truncate(self, *args)
-        
-    def write(self, *args):
-        self._modified = True
-        if not self._writing_ok:
-            raise IOError(9, "Bad file descriptor")
-        else:
-            return StringIO.StringIO.write(self, *args)
-
-    def writelines(self, *args):
-        self._modified = True
-        if not self._writing_ok:
-            raise IOError(9, "Bad file descriptor")
-        else:
-            return StringIO.StringIO.writelines(self, *args)
-
-    def is_modified(self):
-        " Return true if the file has been written to, or truncated"
-        return self._modified
-        
-    def flush(self):
-        " Update the contents of the filespace with the new data "
-        self._filespace.updatefile(self._filename, self._existing_data+self.getvalue())
-        return StringIO.StringIO.flush(self)
-
-    def close(self):
-        " Flush the file and close it "
-        self.flush()
-        self._filespace.closefile(self._filename)
-        return StringIO.StringIO.close(self)
-
 ##def get_function(filename, function_name):
 ##	import compiler
 ##	mod = compiler.parseFile(filename)

